//
// Spherical Harmonic Drop, function calculated in Vertex Shader
//
#ifdef COMPILEVS

#ifdef GL3
#define varying out
#define attribute in
#define texture2D texture
#endif
// precision highp float;
uniform mat4 MVPMatrix;
uniform mat3 NMatrix;
uniform vec4 coef_re[4];
uniform vec4 coef_im[4];
uniform sampler2D phaseColorTex;
#define M_PI          (3.1415926535897932384626433832795)
#define Diffuse (1.)
#define Ambient (0.2)
attribute vec2 posA;
varying highp vec3 eyeNormal;
varying lowp vec3 DiffuseMaterial;
varying lowp vec3 AmbientMaterial;

// Spherical phi,theta,radius -> xyz
vec3 sph2xyz(highp vec2 phith, float r)
{
  return vec3(r * sin(phith.y) * cos(phith.x),
              r * sin(phith.y) * sin(phith.x),
              r * cos(phith.y));
}

// calculate the value of the spherical harmonic at phith
highp vec2 SH(vec2 phith)
{
  const float sqrt_pi = sqrt(M_PI);
  const float fact1 = 0.5 / sqrt_pi;
  const float fact2 = sqrt(3. / (4. * M_PI));
  const float fact3 = 0.25 * sqrt(5. / M_PI);
  const float fact4 = 0.50 * sqrt(15. / M_PI);
  const float fact5 = 0.25 * sqrt(7. / M_PI);
  const float fact6 = 0.25 * sqrt(21. / (2. * M_PI));
  const float fact7 = 0.25 * sqrt(105. / M_PI);
  const float fact8 = 0.25 * sqrt(35. / (2. * M_PI));
  float phi = phith.x;
  float theta = phith.y;
  float z = cos(theta);
  float vtr = sin(theta);
  float cphi = cos(phi);
  float sphi = sin(phi);
  float x = vtr * cphi;
  float y = vtr * sphi;
  float xx = x*x, yy = y*y, zz = z*z;
  highp vec2 val = vec2(0.,0.);
#ifdef S00
  val += S00 * fact1;
#endif
#ifdef S11m
  val += S11m * fact2 * y;
#endif
#ifdef S10
  val += S10 * fact2 * z;
#endif
#ifdef S11p
  val += S11p * fact2 * x;
#endif
#ifdef S22m
  val += S22m * fact4 * x * y;
#endif
#ifdef S21m
  val += S21m * fact4 * y * z;
#endif
#ifdef S20
  val += S20 * fact3 * (-xx - yy + 2. * zz);
#endif
#ifdef S21p
  val += S21p * fact4 * z * x;
#endif
#ifdef S22p
  val += S22p * fact4 / 2. * (xx - yy);
#endif
#ifdef S33m
  val += S33m * fact8 * (3. * xx - yy) * y;
#endif
#ifdef S32m
  val += S32m * fact7 * 2. * x * y * z;
#endif
#ifdef S31m
  val += S31m * fact6 * y * (4. * zz - xx - yy);
#endif
#ifdef S30
  val += S30 * fact5 * z * (2. * zz - 3. * xx - 3. * yy);
#endif
#ifdef S31p
  val += S31p * fact6 * x * (4. * zz - xx - yy);
#endif
#ifdef S32p
  val += S32p * fact7 * (xx - yy) * z;
#endif
#ifdef S33p
  val += S33p * fact8 * (xx - 3. * yy) * x;
#endif
  return val;
}

void main()
{
  // the quality of the drop normal is highly sensitive to this
  // epsilon: too large and the normals are bad, too small and there
  // are spots, too really really too small and there are shimmering
  // smudges due to FP precision limits.
  const float epsilon = 0.5 / 180. * M_PI;
  vec2 posB = vec2(posA.x, posA.y + 2.*epsilon);
  vec2 posC = vec2(posA.x + epsilon, posA.y);

  highp vec2 radA = SH(posA);
  highp vec2 radB = SH(posB);
  highp vec2 radC = SH(posC);
  vec3 posR = sph2xyz(posA, length(radA));
  vec3 pos1 = sph2xyz(posB, length(radB));
  vec3 pos2 = sph2xyz(posC, length(radC));

  vec3 t1 = pos1 - posR;
  vec3 t2 = pos2 - posR;
  vec3 normal =  normalize(cross(t1, t2));

  // output to fragment shader
  eyeNormal = NMatrix * normal;

  // set the output position
  gl_Position = MVPMatrix * vec4(posR, 1.0);

  // set outputs to the fragment shader
  vec2 rad = normalize(radA);
  float unitPhase = atan(rad.y, rad.x) / (2.0 * M_PI);
  lowp vec4 color = texture2D(phaseColorTex, vec2(unitPhase + 0.5, 0));
  DiffuseMaterial = color.xyz * Diffuse;
  AmbientMaterial = color.xyz * Ambient;
}

#endif // COMPILEVS

//
// DropS fragment shader
//
#ifdef COMPILEFS

#ifdef GL3
#define varying in
out vec4 fragData[1];
#define gl_FragColor fragData[0]
#endif

#define SHININESS (20.)

varying highp vec3 eyeNormal;
// The interpolated vertex color for this fragment
varying lowp vec3 DiffuseMaterial;
varying lowp vec3 AmbientMaterial;
uniform lowp float alpha;

void main()
{
    lowp vec3 SpecularMaterial = vec3(0.8);
    highp vec3 Normal = normalize(eyeNormal);
    highp vec3 Light = normalize(vec3(1,0,1));
    highp vec3 E = vec3(0,0,1); // eye?
    highp vec3 H = normalize(Light + E);

    highp float df = max(0.0, dot(Normal, Light));
    highp float sf = max(0.0, dot(Normal, H));

    sf = pow(sf, SHININESS);
    lowp vec3 col = AmbientMaterial + DiffuseMaterial * df + SpecularMaterial * sf;
    // Set the output color to the texture color
    gl_FragColor = vec4(col, alpha);
}

#endif // COMPILEFS
